#!/usr/local/bin/python

import ABAC

import sys
import os
import os.path
from optparse import OptionParser

from util import create_id, creddy_linking_role, creddy_assign, \
    init_principal_identities


class options(OptionParser):
    '''
    Simple option parser that parses --debug
    '''
    def __init__(self):
	OptionParser.__init__(self)
	self.add_option('--debug', action='store_true', dest='debug',
		default=False, help='Debugging output')

def init_clearinghouse(p, debug=False):
    '''
    Clearinghouses don't have any rules. :-)
    '''
    pass

def init_slice_authority(sa, roots, debug=False):
    '''
    Initialize the slice authority.  For each root create a rule that
    recognizes that is a clearinghouse (rule 2 in the document).  The other
    rules are numbered as in the document.
    '''
    for i, r in enumerate(roots):
	creddy_assign(sa, 'clearinghouse', r, '%s/root%d.der' % (sa,i),
		debug)

    creddy_linking_role(sa, 'clearinghouse', sa, 
	    'clearinghouse', 'clearinghouse', '%s/rule1.der' % sa,
	    debug)
    creddy_linking_role(sa, 'GetCredential', sa, 
	    'clearinghouse', 'GetCredential', '%s/rule3.der' % sa,
	    debug)
    creddy_linking_role(sa, 'GetKeys', sa, 
	    'clearinghouse', 'GetCredential', '%s/rule4.der' % sa,
	    debug)
    creddy_linking_role(sa, 'Register_slice', sa, 
	    'clearinghouse', 'Register_slice', '%s/rule5.der' % sa,
	    debug)
    creddy_linking_role(sa, 'Resolve', sa, 
	    'clearinghouse', 'Resolve', '%s/rule6.der' %sa,
	    debug)
    creddy_linking_role(sa, 'DiscoverResources', sa, 
		'clearinghouse', 'ListComponents', '%s/rule7.der' % sa,
		debug)

def init_aggregate_manager(am, roots, debug=False):
    '''
    Again, direct creation of the rules in the document.
    '''
    for i, r in enumerate(roots):
	creddy_assign(am, 'slice_authority', r, '%s/root%d.der' % (am, i),
		debug)
    creddy_linking_role(am, 'slice_authority', am, 
	    'slice_authority', 'slice_authority', '%s/rule1.der' % am,
	    debug)
    creddy_linking_role(am, 'ListResources', am, 
	    'slice_authority', 'DiscoverResources', '%s/rule3.der' % am,
	    debug)
    creddy_linking_role(am, 'CreateSliver', am, 
		'slice_authority', 'CreateSliver', '%s/rule4.der' % am,
		debug)

# Here's the main line
opts, args = options().parse_args()

init_principal_identities(('CH', 'SA', 'AM', 'P'), opts.debug)
init_clearinghouse('CH', opts.debug)
init_slice_authority('SA', ['CH'], opts.debug)
init_aggregate_manager('AM', ['SA'], opts.debug)
