#!/usr/local/bin/python

import ABAC
import sys
import os
from util import load_directory, print_creds, untranslate_cred, \
	translate_cred, id_map
from optparse import OptionParser

# Takes --attr and --principal arguments and directories containing principal's
# credentials.  Loads those credentials into a prover and trys to prove the
# given principal has the given attribute.  The principal names are the
# symbolic names (principal is the same as the directory enclosing it) and so
# is the output, but internally all the credentials use keyids (of course).
# Output is True or false, and a proof if one exists.  All subdirectories are
# considered potential principals, and scanned.

class options(OptionParser):
    '''
    Parse the options
    '''
    def __init__(self):
	OptionParser.__init__(self)
	self.add_option('--principal', dest='principal', 
		help='Calling principal')
	self.add_option('--attr', dest='attr', help='target attribute')
	self.add_option('--debug', action='store_true', dest='debug',
		default=False, help='debugging output')

name_to_id = {} 
opts, args = options().parse_args()

# Load up the prover
context = ABAC.Context()
for p in args:
    load_directory(context, p)

# Read the subdirs looking for identities
for d in [ d for d in os.listdir(".") if os.path.isdir(d)]:
    id, name = id_map(d)
    if id is not None:
	name_to_id[name] = id

# These objects translate to keyids (untrans) or back (trans).
untrans = untranslate_cred(name_to_id)
trans = translate_cred(name_to_id)

# Dump the context if debug is given
if opts.debug:
    print '---'
    print_creds(context.credentials(), trans)
    print '---'

attr, principal = untrans(opts.attr), untrans(opts.principal)
if not all((attr, principal)):
    sys.exit('Usage: %s --principal=x --attr=y dirs\n' + \
	    '--principal and --attr required')

# Here's the proof
ok, certs = context.query(attr, principal)

# Print results
print ok
if ok:
    print_creds(certs, trans)
